#!/bin/sh
source /etc/conf.d/vdr
#set -x
SVDRPCMD=/usr/bin/svdrpsend.pl
NVRAMCMD=/usr/bin/nvram-wakeup
NVRAMCFG="/_config/install/nvram-wakeup.conf"
PROC_ALARM="/proc/acpi/alarm"
# processes which are stopping a shutdown
CMD_LST="make gcc cc1 transcode vcdimager mencoder mplex dvdauthor tosvcd lame vdrsync.pl vdrburn.sh tcmplex tcmplex-panteltje vdr2ac3.sh vdr2dvd.sh vdr2divx.sh"
CTRLFILE="${VIDEO}/shutdown-enabled"
WAKEUP_FILE="${VIDEO}/~wakeup"
NOAD_SCRIPT="${VIDEO}/noad.sh"
# Minimaler Abstand zu naechsten Timer in Minuten
WAKEUP_DIFF_MINIMUM=15
NEXT_TIMER=$1

NT=""
POWEROFF_KERNEL=0

err_exit() {
   if [ ! -f $CTRLFILE ] ; then
      logger -s "Shutdown abgebrochen <$*>"
      $SVDRPCMD MESG "Shutdown abgebrochen"
      $SVDRPCMD MESG "$*"
      exit
   else
      logger -s "Shutdown trotz <$*>"
   fi
}

set_clock() {
   if [ "$DIRECTISA_RTC" = "1" ] ; then
      DIR_ISA="--directisa"
   else
      DIR_ISA="" 
   fi     
   hwclock $DIR_ISA --systohc --localtime --noadjfile
   rm -rf /etc/adjtime > /dev/null 2>&1
}

set_nvram_wakeup() {
   logger -s "Next nvram alarm at: $(date -d "1970-01-01 UTC $1 seconds" +"%Y-%m-%d %R")"
   echo "Using nvram wakeup"
   if [ "$NVRAM_BOARD" != "" ] ;then
      if [ -f "$NVRAMCFG.$NVRAM_BOARD" ] ;then
         NVRAMCMD="$NVRAMCMD -C $NVRAMCFG.$NVRAM_BOARD"
      else
         NVRAMCMD="$NVRAMCMD -I $NVRAM_BOARD"	    
      fi    
   fi
   [ "$DIRECTISA_NVRAM" = "1" ] && NVRAMCMD="$NVRAMCMD --directisa"

   $NVRAMCMD -ls $1
   [ $? -eq 2 ] && err_exit "nvram-wakeup fehlgeschlagen"
   [ $? -eq 1 ] && POWEROFF_KERNEL=1
}

set_acpi_timer() {
   if [ -f $PROC_ALARM ] && [ $1 -gt 0 ] ; then
      # check if NEXT_TIMER is within next 24 hours
      WAKEUP=$(($(date -d "1 day" +%s)-120))
      if [ $1 -lt $WAKEUP ] ; then
         WAKEUP=$1
      else
         NEXT_TIMER=$WAKEUP
      fi
      NEXT_ALARM=$(date -d "1970-01-01 UTC $WAKEUP seconds" +"%Y-%m-%d %R")
      logger -s "Next acpi alarm at: $NEXT_ALARM"
      echo -n "$NEXT_ALARM" > $PROC_ALARM
      sleep 2
      echo -n "$NEXT_ALARM" > $PROC_ALARM
   else
      [ $1 -gt 0 ] && err_exit "ACPI wakeup nicht moeglich"
   fi
}

logger -s "$0 $*"
[ -f $WAKEUP_FILE ] && rm -f $WAKEUP_FILE 
[ -f /video/noad.sh ] && rm /video/noad.sh

# check for forced Shutdown
[ -f /tmp/sandboxpids.tmp ] && err_exit "emerge laeuft"
# check if there are some users logged in
if [ "$IGNORE_USER" = "0" ] ; then
   USERS=$(who | wc -l)
   [ "$USERS" != "0" ] && err_exit "$USERS Benutzer sind angemeldet"
fi
for x in ${CMD_LST} ; do
   [ "$(pidof $x)" != "" ] && err_exit "$x laeuft"
done
# check for freevo
[ -f /_config/status/~vdrdisabled ] && err_exit "VDR ist nicht aktiv"

ACT_DATE=$(date +%s)

# Naechsten Timer pruefen
if [ $1 -gt $WAKEUP_RESERVE ] ; then
   NEXT_TIMER=$(($1 - $WAKEUP_RESERVE))
   WAKEUP_DIFF=$((($NEXT_TIMER - $ACT_DATE) / 60))
   [ $WAKEUP_DIFF -lt $WAKEUP_DIFF_MINIMUM ] && err_exit "Naechster Timer in $WAKEUP_DIFF Minuten"
else
   NEXT_TIMER=0
fi

DAILY_WAKEUP=0

# Ist taeglicher Wakeup aktiviert
if [ "$WAKEUP_DURATION" != "" -a $WAKEUP_DURATION -gt 0 ] ; then
   ACT_HOUR=$(date +%k)
   if [ $WAKEUP_HOUR -gt $ACT_HOUR ] ; then
      NEXT_WAKEUP=$(date -d "$WAKEUP_HOUR:00" +"%s")
   else
      NEXT_WAKEUP=$(date -d "$WAKEUP_HOUR:00 1 day" +"%s")
   fi
   NEXT_WAKEUP=$(($NEXT_WAKEUP - $WAKEUP_RESERVE))
   WAKEUP_DIFF=$((($NEXT_WAKEUP - $ACT_DATE) / 60))
   [ $WAKEUP_DIFF -lt $WAKEUP_DIFF_MINIMUM ] && err_exit "Taeglicher Wakeup in $WAKEUP_DIFF Minuten"
   if [ "$NEXT_TIMER" = "0" ] || [ $NEXT_TIMER -gt $NEXT_WAKEUP ] ; then
      NEXT_TIMER=$NEXT_WAKEUP
      DAILY_WAKEUP=1
   fi
fi

# Uhrzeit setzen
set_clock

if [ "$WAKEUP_METHOD" = "NVRAM" ] ; then
   set_nvram_wakeup $NEXT_TIMER
elif [ "$WAKEUP_METHOD" = "ACPI" ] ; then
   set_acpi_timer $NEXT_TIMER
else
   logger -s "No wakeup possible - shutting down anyway"
fi

# check for noad 
ps x | grep -m 1 noad | grep " after "| sed -e "s|.*/usr/bin/noad|/usr/bin/noad|" -e "s/ after / after \"/" -e "s/$/\"/" > $NOAD_SCRIPT
[ "$WAKEONLAN" = "1" ] && ethtool -s eth0 wol g
if [ $1 != 0 ] ; then
   NT=$(date -d "1970-01-01 UTC $NEXT_TIMER seconds" +"%Y%m%d%H%M")
   echo "$1;$3;$4" > $WAKEUP_FILE
   touch -t $NT $WAKEUP_FILE
fi

if [ "$POWEROFF_KERNEL" = "1" ] || [ "$SHUTDOWN_METHOD" = "PowerOffKernel" ]; then
   lilo -R poweroff
   SHUTDOWN_CMD="shutdown -r now"
else
   if [ "$ACPI" = "1" ] && [ "$SHUTDOWN_METHOD" = "STR" ]; then
      DOCHECK="0"
      if [ "$FSCK_WEEK" != "0" ] ; then
         LCHKF="/etc/lastCheck"
         if [ ! -z $LCHKF ] && [ -f $LCHKF ] ; then
            LCHECKED=`ls --full-time $LCHKF | cut -f 7 -d " "`
            lWeek=`date -d "$LCHECKED" +"%Y%W"`
            actWeek=`date +"%Y%W"`
            if [ $(($actWeek-$lWeek)) -ge $FSCK_WEEK ] ; then
               DOCHECK="1"
            fi
         else   
            DOCHECK="1"
         fi
      fi
      if [ "$DOCHECK" = "0" ] ;then
         SHUTDOWN_CMD="/_config/bin/hibernate.sh"
      else
         SHUTDOWN_CMD="halt"
      fi    
   elif [ "$SHUTDOWN_METHOD" = "Shutdown" ]; then
      SHUTDOWN_CMD="shutdown -h now"
   else
      SHUTDOWN_CMD="halt"
   fi
fi
logger -s "Shutting down: <$SHUTDOWN_CMD>"
/_config/bin/activy_lcdtimer.sh
$SVDRPCMD MESG "System wird heruntergefahren"
nohup $SHUTDOWN_CMD &
rm -f $CTRLFILE
